/******************************************************************************
*
* project name:    TICHESS
* project version: 1.10
* file name:       tichess.c
* file version:    1.07
* initial date:    20/02/2000
* author:          thomas.nussbaumer@peem.com
* description:     the entry point and the splashscreen stuff
*
* -----------------------------------------------------------------------------
* Revision History
* -----------------------------------------------------------------------------
* v1.05: 07/03/2000
*        (1) modifications for DOORS version
*        (2) modifications for batch building
*        (3) splashscreen modified
*        (4) some modifications for the dialogs
*
* v1.06: 08/03/2000
*        HW checking routine from Julien Muchembled integrated
*        now showing web address on splashscreen
*
* v1.07: 10/03/2000
*        Splashscreen now monochrome
*        using NEW gray.o and gray.h (HW autodetect and no problems on HW2)
*        
******************************************************************************/

//-----------------------------------------------------------------------------
// this define is for batch compiling where the defines are specified
// at the commandline
//-----------------------------------------------------------------------------
#ifndef AUTO_BUILD

//=============================================================================
// set the following define to 1 if you want the DOORS OS version
//
// the DOORS OS version uses gray4lib for the grayscale GFX which looks much
// better on HW2 calculators. Additionally the checking for the HW version
// can be ommitted because the kernel will do it by itself 
//=============================================================================
#define COMPILE_FOR_DOORS 0

//=============================================================================
// THE FOLLOWING PRECOMPILER DEFINES ARE USED TO SET A SPECIFIC CALCULATOR TYPE
//
// !!!!!! NOTE THAT ONLY ONE OF THE DEFINES MAY BE SET TO '1' !!!!!
//=============================================================================
#define TI89_VERSION      0    // generate version for TI89
#define TI92_VERSION      1    // generate version for TI92

#endif


#if COMPILE_FOR_DOORS
   #include <doors.h>
   #include "krnlgray.h"
#else
   #include <nostub.h>
#endif

#include <graph.h>
#include <kbd.h>
#include <alloc.h>
#include <vat.h>
#include <string.h>
#include <args.h>
#include <ctype.h>
#include <setjmp.h>
#include "gray.h"        // new gray.h !!!
#include "tichess.h"

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// due to a very strange behavior of TIGCC we cannot use separate object until
// yet (the globals are not initialized correctly - NOT even the constants !!)
//
// So we've to go that really silly "include all into one" way ...
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#include "gui.c"      // gui stuff
#include "engine.c"   // engine stuff

#if TI92_VERSION
int _ti92plus;        // needed to produce ti92plus code
#else 
int _ti89;            // needed to produce ti89 code
#endif

//--------------------------------------------------
// declarations of the necessary "external" routines
//--------------------------------------------------
int  InitMemory();
void FreeMemory();
void MainGUI(int level);


/*===========================================================================*/
/* returns HW version (1 for HW1 calculators)                                */
/*                                                                           */
/* Thanx to Julien Muchembled for the code                                   */
/*===========================================================================*/

// not used anymore due to new gray.o

#if 0
unsigned long GetHardwareVersion()
{
    unsigned long hwpb, *rombase;
    rombase = (unsigned long *)((*(unsigned long *)0xC8) & 0x600000);
    hwpb = rombase[65];
    return (hwpb - (unsigned long)rombase  < 0x10000
            && *(unsigned short *)hwpb > 22 ?
            *(unsigned long *)(hwpb + 22) : 1);
}
#endif

/*===========================================================================*/
/* draws the splashscreen                                                    */
/*                                                                           */
/* NOTE: not grayscale                                                       */
/*===========================================================================*/
void SplashScreen(void) {
    int i,j;
    int x00,y00,x01,y01;
    int x00_o,y00_o,x01_o,y01_o;
    int maxx    = ScrRect->xy.x1;
    int maxy    = ScrRect->xy.y1;
    int deltax0 = -1;
    int deltay0 = -1;
    char*    s1 = "The TI-Chess Team";
    char*    s2 = "proudly presents";
    char*    s3 = "TI-Chess v1.10";
    char*    s4 = "http://members.xoom.com/tnussb2";
    int      px1,px2,px3,px4;
    int      py1,py2,py3,py4;

    // set initial line endpoints
    x00 = maxx/3;
    y00 = maxy/4;
    x01 = maxx*2/3;
    y01 = maxy/2;

    px1 = (ScrRect->xy.x1-DrawStrWidth(s1,F_8x10))/2;
    px2 = (ScrRect->xy.x1-DrawStrWidth(s2,F_6x8))/2;
    px3 = (ScrRect->xy.x1-DrawStrWidth(s3,F_8x10))/2;
    px4 = (ScrRect->xy.x1-DrawStrWidth(s4,F_4x6))/2;
    py1 = ScrRect->xy.y1/4;
    py2 = ScrRect->xy.y1/2;
    py3 = ScrRect->xy.y1*3/4;
    py4 = ScrRect->xy.y1-5;

    for (i=0;i<10000;i++) {
        // draw the text
        FontSetSys(F_8x10);
        DrawStr(px1,py1,s1,A_REPLACE);
        DrawStr(px3,py3,s3,A_REPLACE);
        FontSetSys(F_6x8);
        DrawStr(px2,py2,s2,A_REPLACE);
        FontSetSys(F_4x6);
        DrawStr(px4,py4,s4,A_REPLACE);
        // draw the line
        DrawLine(x00,y00,x01,y01,A_NORMAL);
        // keep a backup of the endpoint coordinates for later removing
        x00_o =  x00;
        x01_o =  x01;
        y00_o =  y00;
        y01_o =  y01;
        
        // add delta values to the endpoint coordinates
        x00 += deltax0;
        x01 += deltax0;
        y00 += deltay0;
        y01 += deltay0;
        
        // if an endpoint reaches the border, invert the corresponding delta
        // value
        if (x00 <= 1 || x01 <= 1 || x00 >= maxx-1 || x01 >= maxx-1) {
           deltax0 = -deltax0;
        }
        if (y00 <= 1 || y01 <= 1 || y00 >= maxy-1 || y01 >= maxy-1) {
           deltay0 = -deltay0;
        }
        
        // check for key pressed
        if (kbhit()) {
            ngetchx();
            break; 
        }

        // clear almost all of the previous drawn line
        DrawLine(x00_o-1,y00_o-1,x01_o-1,y01_o-1,A_XOR);
    }
}


/*===========================================================================*/
/* MAIN - where all the fun starts ...                                       */
/*===========================================================================*/
void _main(void) {
    ESI        argptr;
    int        argtype;
    int        level = 2;
    int        i;
    char*      s1 = "ERROR: Memory";
    char*      s2 = "Insufficient Memory.";
    char*      s3 = "(Req'd around 25 kB)";

#ifndef DOORS
    LCD_BUFFER buffer;
    LCD_save(buffer);
#endif

    ClrScr();

    if (InitMemory() == FALSE) {
        FreeMemory();

        FontSetSys(F_8x10);
        i = DrawStrWidth(s1,F_8x10); 
        DrawStr((ScrRect->xy.x1-i)/2,ScrRect->xy.y1/3,s1,A_REPLACE);
        FontSetSys(F_6x8);
        i = DrawStrWidth(s2,F_6x8); 
        DrawStr((ScrRect->xy.x1-i)/2,ScrRect->xy.y1/2,s2,A_REPLACE);
        FontSetSys(F_6x8);
        i = DrawStrWidth(s3,F_6x8); 
        DrawStr((ScrRect->xy.x1-i)/2,ScrRect->xy.y1*2/3,s3,A_REPLACE);
        ngetchx();
#ifndef DOORS
        LCD_restore(buffer);
#endif       
        return;
    }

    SplashScreen();

#ifndef DOORS
    if (!GrayMode(GRAY_ON)) {
       LCD_restore(buffer);
       return;
    }
#else
    if (!GrayMode(GRAY_ON)) return;
#endif
        

    // evaluate "level" commandline parameter if given
    InitArgPtr(argptr);
    if ((argtype=GetArgType(argptr))!=END_TAG) {
        if (argtype==POSINT_TAG) {
          level=(int)GetIntArg(argptr);
          level++;
          if (level > MAX_USERLEVEL) level = MAX_USERLEVEL;
          if (level < MIN_USERLEVEL) level = MIN_USERLEVEL;
        }
    }

#if DO_STATISTICS
    InitStatistics();
#endif

    MainGUI(level);
    GrayMode(GRAY_OFF);
    FreeMemory();
#ifndef DOORS    
    LCD_restore(buffer);
#endif
}

//##########################################################################
//###################### NO MORE FAKES BEYOND THIS LINE ####################
//##########################################################################
