/******************************************************************************
*
* project name:    TICHESS
* project version: 1.10
* file name:       tichess.h
* file version:    1.02
* initial date:    20/02/2000
* author:          thomas.nussbaumer@peem.com
* description:     project header file
*
* -----------------------------------------------------------------------------
* Revision History
* -----------------------------------------------------------------------------
*
* v1.00: 20/02/2000
*        initial version
*
* v1.01: 02/03/2000
*        TI89 support
*        HW platform now define-able
*
* v1.02: 06/03/2000
*        calculator defines moved to tichess.c
*  
******************************************************************************/

#ifndef TICHESS_H   
#define TICHESS_H   


#if !TI89_VERSION
#if !TI92_VERSION
#error: Neither "TI89_VERSION" nor "TI92_VERSION" defined
#endif
#endif

#if TI89_VERSION
#if TI92_VERSION
#error: Cannot define simultaneous "TI89_VERSION" and "TI92_VERSION"
#endif
#endif


#define DO_STATISTICS     0    // calculate statistics and print them 
#define CHECK_PAWN_STRUCT 0    // check one-the-fly calculated pawn variables


#define MAX_USERLEVEL 5
#define MIN_USERLEVEL 2

/*---------------------------------------------------------------------------*/
/*  generic constants                                                        */
/*---------------------------------------------------------------------------*/
#define BOARD_SIZE   120     /* dimension of extended chess board            */
#define MAX_DEPTH     20     /* maximum search depth                         */
#define NR_DIRS       16     /* number of directions of all figures          */
#define NR_FIGS        7     /* number of different figures (including NONE) */
#define STACK_SIZE  1000     /* size of move stack                           */

/*---------------------------------------------------------------------------*/
/*  figure constants                                                         */
/*---------------------------------------------------------------------------*/
#define B_KING     -6          /* black figures      */
#define B_QUEEN    -5
#define B_KNIGHT   -4
#define B_BISHOP   -3
#define B_ROOK     -2
#define B_PAWN     -1
#define EMPTY       0          /* empty field        */
#define W_PAWN      1          /* white figures      */
#define W_ROOK      2 
#define W_BISHOP    3
#define W_KNIGHT    4
#define W_QUEEN     5
#define W_KING      6
#define OUTSIDE   100          /* outside real board */ 

/*---------------------------------------------------------------------------*/
/*  figure values                                                            */
/*---------------------------------------------------------------------------*/
#define VALUE_PAWN        100
#define VALUE_ROOK        500
#define VALUE_BISHOP      350
#define VALUE_KNIGHT      325
#define VALUE_QUEEN       900
#define VALUE_KING          0 
#define VALUE_MATT      32000
#define MAX_POS         32000

/*---------------------------------------------------------------------------*/
/*  bonus values for main variant and killer moves                           */
/*---------------------------------------------------------------------------*/
#define HVARBONUS         500
#define KILLER1BONUS      250
#define KILLER2BONUS      150

/*---------------------------------------------------------------------------*/
/*  maximum value of materials                                               */
/*---------------------------------------------------------------------------*/
#define VALUE_MAX     ((4*(VALUE_ROOK+VALUE_BISHOP+VALUE_KNIGHT))+(2*VALUE_QUEEN))
#define VALUE_ENDGAME (4*VALUE_ROOK+(2*VALUE_BISHOP))

/*---------------------------------------------------------------------------*/
/* field names for often used fields (increasing readability)                */
/*---------------------------------------------------------------------------*/
#define  A1    21
#define  B1    22
#define  C1    23
#define  D1    24
#define  E1    25
#define  F1    26
#define  G1    27
#define  H1    28
#define  C2    33
#define  H2    38
#define  A3    41
#define  C3    43
#define  D3    44
#define  E3    45
#define  A6    71
#define  C6    73
#define  D6    74
#define  E6    75
#define  A7    81
#define  C7    83
#define  H7    88
#define  A8    91
#define  B8    92 
#define  C8    93
#define  D8    94
#define  E8    95
#define  F8    96
#define  G8    97
#define  H8    98

/*---------------------------------------------------------------------------*/
/* line and column names                                                     */
/*---------------------------------------------------------------------------*/
#define A_COLUMN   1
#define B_COLUMN   2
#define C_COLUMN   3
#define D_COLUMN   4
#define E_COLUMN   5
#define F_COLUMN   6
#define G_COLUMN   7
#define H_COLUMN   8
#define ROW_1      2
#define ROW_2      3
#define ROW_3      4
#define ROW_4      5
#define ROW_5      6
#define ROW_6      7
#define ROW_7      8
#define ROW_8      9

/*---------------------------------------------------------------------------*/
/* rochade numbers                                                           */
/*---------------------------------------------------------------------------*/
#define ROCHADE_NONE  0 
#define ROCHADE_SHORT 1
#define ROCHADE_LONG  2

/*---------------------------------------------------------------------------*/
/* color numbers                                                             */
/*---------------------------------------------------------------------------*/
#define WHITE    1
#define BLACK   -1

/*---------------------------------------------------------------------------*/
/* logic values                                                              */
/*---------------------------------------------------------------------------*/
#define TRUE           1
#define FALSE          0
#define LEGAL          1
#define ILLEGAL        0
#define OK             1

/*---------------------------------------------------------------------------*/
/* data type for move stack                                                  */
/*---------------------------------------------------------------------------*/
typedef struct {
   int from;         /* from field             */
   int to;           /* to field               */
   int killed_fig;   /* killed figure          */
   int promoted_fig; /* promoted figure        */
   int rochade_nr;   /* kind of rochade        */
   int ep_field;     /* enpassant field        */
   int value;        /* value for move sorting */
} move_t;

/*---------------------------------------------------------------------------*/
/* index of figures in offset list and long/short marker (used by move gen.) */
/*---------------------------------------------------------------------------*/
typedef struct {
   int start;
   int end;
   int longmove;
} figoffset_t; 

/*---------------------------------------------------------------------------*/
/* information about pawn/figure constellation                               */
/*---------------------------------------------------------------------------*/
typedef struct {
   int white;
   int black;
   int diff;
} bicolor_t;

/*---------------------------------------------------------------------------*/
/* information about from/to field                                           */
/*---------------------------------------------------------------------------*/
typedef struct {
   int from;
   int to;
} fromto_t;

/*---------------------------------------------------------------------------*/
/* data structure for killer moves                                           */
/*---------------------------------------------------------------------------*/
typedef struct {
   fromto_t killer1;
   fromto_t killer2;
} killer_t;

#endif  


